//******************************************************************************
//******************************************************************************
/**
 * @file    IClientProperties.h
 * @author  Markus van Pinxteren
 * @date    20.03.2014
 *
 * @brief   Interface for client properties
*/
//******************************************************************************
//******************************************************************************
#ifndef __ICLIENTPROPERTIES_H__
#define __ICLIENTPROPERTIES_H__


#include "BPNGDefines.h"

/// The IClientProperties interface replaces the deprecated \a ClientProperties struct
/**
 Call IBPNGClient::getClientProperties() to get a pointer to an instance of this interface class.
*/
struct IClientProperties
{
    /**
     @brief Set Common properties
     @param nameOfTester Name of tester that is written to the converted file names
     @param maxOutputSizeMB Maximum file size for converted files. When this size is reached a new file is created.
     @param separatedTimeFormat Specifies the time format that should be used for converted files. Set 1 for long format (e.g. [2011-12-20]_10.15.48) or 0 for short frmat (e.g. 20111220_101548)
     @param separatedTimeFormatInOfflineSet Specifies the time format that should be used for offline conversion sets. Set 1 for long format (e.g. [2011-12-20]_10.15.48) or 0 for short format (e.g. 20111220_101548)
     @param alternativeLoggerName The logger device's name is included in the converted files' names. An alternative logger name can be used.
     @param useAlternativeLoggerName Set this fied to 1 if the alternative logger name should be used in converted file names, 0 if not.
     @param useSubDirectories Set to 1 if converted files should be stored in subdirectories named be their start date, set 0 if they should not.
     @param midnightSplitting Set to 1 if converted files should be splitted at 00:00:00 of each date, set to 0 if they should not.
     @param fileTimeSpansLikeSelection The file names of the converted files contain the time span of the included data.
	                               Setting this parameter to 1 will create time spans like they were specified in the IConversionSet.
	                               Setting this to 0 will create time spans according to the effectively included data.
     @param markerNumberInFileNames Specifies whether the indices of the marker included in a converted file should be appended to its file name
     @param subfolderWithLoggerName Specifies whether the name of the subfolder the converted files are stored in should contain the logger name or not.
     @param maxOutputSizeMBSortedDownload Maximum file size for sorted download trace files. When this size is reached a new file is created.
    */
    virtual void WINAPI setCommonProperties(const char* nameOfTester, int maxOutputSizeMB, BOOL separatedTimeFormat, BOOL separatedTimeFormatInOfflineSet,
        const char* alternativeLoggerName, BOOL useAlternativeLoggerName, BOOL useSubDirectories, BOOL midnightSplitting, BOOL fileTimeSpansLikeSelection,
        BOOL markerNumberInFileNames, BOOL subfolderWithLoggerName, int maxOfflineZipSizeMB, int maxOutputSizeMBSortedDownload, 
        BOOL traceCutterStorage, const char* traceCutterFNPatternConversion, BOOL createOfflineDataOnTraceCutterStorage, const char* traceCutterFNPatternOfflineData) = 0;


    virtual void WINAPI setNameOfTester(const char* name) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setMaxOutputSize(int size) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setSeparatedTimeFormat(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setSeparatedTimeFormatInOfflineSet(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setAlternativeLoggerName(const char* name) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setAlternativeLoggerNameActive(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setConvertedFilesInSubDirsActive(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setMidnightSplittingActive(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setFileTimeSpansLikeSelection(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setMarkerNumbersInFileNames(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setSubfolderWithLoggerName(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setMaxOfflineZipSize(int size) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setMaxOutputSizeSortedDownload(int size) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setTraceCutterStorage(BOOL flag) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setTraceCutterFNPatternConversion(const char* pattern) = 0; ///< see parameter description of \a setCommonProperties()
    virtual void WINAPI setCreateOfflineDataOnTraceCutterStorage(BOOL flag) = 0;
    virtual void WINAPI setTraceCutterFNPatternOfflineData(const char* pattern) = 0; ///< see parameter description of \a setCommonProperties()

    virtual const char* WINAPI getNameOfTester() = 0; ///< see parameter description of \a setCommonProperties()
    virtual int WINAPI getMaxOutputSize() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isSeparatedTimeFormat() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isSeparatedTimeFormatInOfflineSet() = 0; ///< see parameter description of \a setCommonProperties()
    virtual const char* WINAPI getAlternativeLoggerName() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isAlternativeLoggerNameActive() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isConvertedFilesInSubDirsActive() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isMidnightSplittingActive() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isFileTimeSpansLikeSelection() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isMarkerNumbersInFileNames() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isSubfolderWithLoggerName() = 0; ///< see parameter description of \a setCommonProperties()
    virtual int WINAPI getMaxOfflineZipSize() = 0; ///< see parameter description of \a setCommonProperties()
    virtual int WINAPI getMaxOutputSizeSortedDownload() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isTraceCutterStorage() = 0; ///< see parameter description of \a setCommonProperties()
    virtual const char* WINAPI getTraceCutterFNPatternConversion() = 0; ///< see parameter description of \a setCommonProperties()
    virtual BOOL WINAPI isCreateOfflineDataOnTraceCutterStorage() = 0;
    virtual const char* WINAPI getTraceCutterFNPatternOfflineData() = 0; ///< see parameter description of \a setCommonProperties()


    /**
     @brief Set CAN pseudo properties for writing time stamp messages
     @param writeTimeStampMsg Active flag for writing periodical CAN pseudo messages with absolute time stamps
     @param channelIndex CAN channel for the time stamp pseudo messages
     @param dlc          DLC for the time stamp pseudo messages
     @param canID        CAN ID for the time stamp pseudo messages
     @param hourBitPos   Bit position for the hour (0..23, 5 bit length) value in the CAN data bytes
     @param minBitPos    Bit position for the minute (0..59, 6 bit length) value in the CAN data bytes
     @param secBitPos    Bit position for the second (0..59, 6 bit length) value in the CAN data bytes
     @param dayBitPos    Bit position for the day (1..31, 5 bit length) value in the CAN data bytes
     @param monthBitPos  Bit position for the month (1..12, 4 bit length) value in the CAN data bytes
     @param yearBitPos   Bit position for the year (8 bit length) value in the CAN data bytes
    */
    virtual void WINAPI setCANPseudoMsgTimeStampProperties(BOOL writeTimeStampMsg, uint32_t channelIndex, uint32_t dlc, uint32_t canID, 
        uint32_t hourBitPos, uint32_t minBitPos, uint32_t secBitPos, uint32_t dayBitPos, uint32_t monthBitPos, uint32_t yearBitPos) = 0;
    
    virtual BOOL WINAPI isCANPseudoMsgTimeStampActive() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgChannelIndexTimeStamp() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgDlcTimeStamp() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgCanIDTimeStamp() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgHourBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgMinBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgSecBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgDayBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgMonthBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()
    virtual uint32_t WINAPI getCANPseudoMsgYearBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTimeStampProperties()

    /**
     @brief Set CAN pseudo properties for writing trigger messages
     @param writeTriggerMessage Active flag for writing CAN pseudo messages with trigger information
     @param channelIndex        CAN channel for the trigger pseudo messages
     @param dlc                 DLC for the trigger pseudo messages
     @param canID               CAN ID for the trigger pseudo messages
     @param triggerNumBitPos    Bit position for the trigger's index (16 bit length)
    */
    virtual void WINAPI setCANPseudoMsgTriggerProperties(BOOL writeTriggerMessage, uint32_t channelIndex, uint32_t dlc, uint32_t canID, uint32_t triggerNumBitPos) = 0;
    virtual BOOL WINAPI isCANPseudoMsgTriggerActive() = 0; ///< see parameter description of \a setCANPseudoMsgTriggerProperties()
    virtual uint32_t WINAPI getCANPseudoMsgChannelIndexTrigger() = 0; ///< see parameter description of \a setCANPseudoMsgTriggerProperties()
    virtual uint32_t WINAPI getCANPseudoMsgDlcTrigger() = 0; ///< see parameter description of \a setCANPseudoMsgTriggerProperties()
    virtual uint32_t WINAPI getCANPseudoMsgCanIDTrigger() = 0; ///< see parameter description of \a setCANPseudoMsgTriggerProperties()
    virtual uint32_t WINAPI getCANPseudoMsgTriggerNumBitPos() = 0; ///< see parameter description of \a setCANPseudoMsgTriggerProperties()

    /**
     @brief Set MOST pseudo properties
     @param active     Active flag for writing MOST pseudo messages for trigger
     @param src        Source address
     @param target     Target address
     @param fktBlockID Function block ID
     @param fktID      Function ID
    */
    virtual void WINAPI setMOSTPseudoMsgProperties(BOOL active, uint32_t src, uint32_t target, uint32_t fktBlockID, uint32_t fktID) = 0;
    virtual BOOL WINAPI isMOSTPseudoMsgActive() = 0; ///< see parameter description of \a setMOSTPseudoMsgProperties()
    virtual uint32_t WINAPI getMOSTPseudoMsgSourceAddr() = 0; ///< see parameter description of \a setMOSTPseudoMsgProperties()
    virtual uint32_t WINAPI getMOSTPseudoMsgTargetAddr() = 0; ///< see parameter description of \a setMOSTPseudoMsgProperties()
    virtual uint32_t WINAPI getMOSTPseudoMsgFktBlockID() = 0; ///< see parameter description of \a setMOSTPseudoMsgProperties()
    virtual uint32_t WINAPI getMOSTPseudoMsgFktID() = 0; ///< see parameter description of \a setMOSTPseudoMsgProperties()

    /// Set whether to use the satellite time stamp in GPS formats instead of the logger time stamp
    virtual void WINAPI useSatelliteTimeForGPSFormats(BOOL flag) = 0;
    /// Returns whether to use the satellite time stamp in GPS formats instead of the logger time stamp
    virtual BOOL WINAPI isSatelliteTimeForGPSFormats() = 0;
    /// Set the channelLabels of the isochronous channels as comma separated string
    virtual void WINAPI setIsochronousMost150Channels(const char* channels) = 0;
    /// Returns the channelLabels of the isochronous channels as comma separated string
    virtual const char* WINAPI getIsochronousMost150Channels() = 0;

    /// Set whether to activate the analogue data to CAN pseudo message feature
    virtual void WINAPI setAnalogToCANPseudoActive(BOOL flag) = 0;

    /**
     @brief Set analog port settings
     @param analogPort Analogue port index
     @param isActive Specifies whether the data of this port should be written to CAN pseudo messages
     @param canChannel Specifies the CAN channel that should be used for the pseudo messages
     @param canID Specifies the CAN ID that should be used for the pseudo messages
     @param dbcPath The path to the DBC file that specifies the signal of the CAN ID's message that should carry the value
    */
    virtual void WINAPI addAnalogPortSettings(uint16_t analogPort, BOOL isActive, uint32_t canChannel, uint32_t canID, const char* dbcPath) = 0;
    /// Clears all port settings set with the \a addAnalogPortSettings() function.
    virtual void WINAPI clearAnalogPortSettings() = 0;
};

#endif // __ICLIENTPROPERTIES_H__