//************************************************************
//
// BPNGLoggerDetector.hh
//
/** @file BPNGLoggerDetector.hh
 *
 * @brief Logger Detector Sample
 */
//************************************************************

#ifndef BPNGLOGGERDETECTOR_HH
#define BPNGLOGGERDETECTOR_HH

#include "IBPNGClient.h"
#include "IBPNGClientListener.h"

#include <vector>
#include <string>
#include <iostream>
#include <sstream>

/**
A simple class that represents a Telemotive System Link chain.
*/
class TSLCluster
{
public:

    /**
    A enumaration of types of which task the connection will be used for.
    */
    enum ConnectionType
    {
        DOWNLOAD,   ///< Download tasks
        CONVERSION, ///< Conversion tasks
        CONFIG,     ///< Configuration tasks
        BUGREPORT,  ///< Create bug report
        FW_UPDATE   ///< make Firmware update
    };

    /**
    Constructor
    */
    TSLCluster()
    {
    }

    /**
    Constructor

    @param firstDevice the first device of the chain.
    */
    TSLCluster(OnlineLoggerInfo firstDevice)
    {
        addDevice(firstDevice);
    }

    /**
    Add a BPNGDevice to the TSLCluster.

    @param device the new device
    */
    void addDevice(OnlineLoggerInfo device)
    {
        if (m_tslChain.empty())
        {
            m_tslNetworkId = device.tslNetworkId;
        }

        m_tslChain.push_back(device);
    }

    /**
    get the device list (ips separated by ';') for IBPNGClient::connectLogger(int numLogger, OnlineLoggerInfo* devices)

    It is recommended to exclude the Remote Control Touch on Download and Conversion
    cause its not a data logging device.

    @param connectionType the type of work for which the connection will be used
    @return the ip string
    */
    std::vector<OnlineLoggerInfo> getConnectionVector(ConnectionType connectionType)
    {
        bool excludeRCT;

        switch (connectionType)
        {
        case DOWNLOAD:
        case CONVERSION:
            excludeRCT = true;
            break;
        case CONFIG:
        case BUGREPORT:
        case FW_UPDATE:
            excludeRCT = false;
            break;
        }

        std::vector<OnlineLoggerInfo> retVal;
        for (size_t i = 0; i < m_tslChain.size(); ++i)
        {
            OnlineLoggerInfo device = m_tslChain[i];
            if (excludeRCT && device.deviceType == DEV_RC_TOUCH)
            {
                continue;
            }
            retVal.push_back(device);
        }

        return retVal;
    }

    /**
    get the number of participants for IBPNGClient::getTSLClient(int numTSLMember)

    It is recommended to exclude the Remote Control Touch on Download and Conversion
    cause its not a data logging device.

    @param connectionType the type of work for which the connection will be used
    @return number of participants
    */
    int getNumMembers(ConnectionType connectionType)
    {

        switch (connectionType)
        {
        case DOWNLOAD:
        case CONVERSION:
            break;
        case CONFIG:
        case BUGREPORT:
        case FW_UPDATE:
            return m_tslChain.size();
        }

        int retVal = 0;
        for (size_t i = 0; i < m_tslChain.size(); ++i)
        {
            OnlineLoggerInfo device = m_tslChain[i];
            if (device.deviceType == DEV_RC_TOUCH)
            {
                continue;
            }

            ++retVal;
        }

        return retVal;
    }

    /**
    Get the name of TSL chain. All devices in the chain have the same TSL name.

    @return the TSL name.
    */
    std::string getTSLName()
    {
        return m_tslChain[0].tslName;
    }

    /**
    Stream the TSLCluster to cout.
    */
    void print()
    {
        for (size_t i = 0; i < m_tslChain.size(); ++i)
        {
            OnlineLoggerInfo device = m_tslChain[i];
            std::cout << "--Dev: [" << device.name << "] IP[" << device.ip << "]\n";
        }
    }

    /**
    Begin iterator for ranged base for loop

    @return the begin iterator of internal BPNGDevice vector.
    */
    std::vector<OnlineLoggerInfo>::iterator begin() { return m_tslChain.begin(); }

    /**
    End iterator for ranged base for loop

    @return the end iterator of internal BPNGDevice vector.
    */
    std::vector<OnlineLoggerInfo>::iterator end() { return m_tslChain.end(); }

private:
    /**
    The internal vector of BPNGDevice in the TSL.
    */
    std::vector<OnlineLoggerInfo> m_tslChain;

    /**
    The network id of the TSL.
    */
    int m_tslNetworkId;
};

/**
A simple minimal implementation for a logger detection. Uses an IBPNGClient instance and the method IBPNGClient::scanNetworkForLogger().
Sets itselfs as IBPNGClientListener to the IBPNGClient instance (IBPNGClient::addListener(IBPNGClientListener* listener)).
The callbacks onBPNGDeviceDetected(OnlineLoggerInfo *info), onBPNGDeviceDisappeared(OnlineLoggerInfo *info)
and onBPNGDeviceStateChange(OnlineLoggerInfo *info) informs the class about the device states in network.
*/
class BPNGLoggerDetector : public IBPNGClientListener
{
public:
    /**
    Constructor
    */
    BPNGLoggerDetector()
    {
    }

    ~BPNGLoggerDetector()
    {
        for (size_t i = 0; i < m_loggerList.size(); ++i)
        {
            freeCopy(m_loggerList[i]);
        }

        m_loggerList.clear();
    }

    /**
    Returns a vector of detected BPNGDevice in local networks.

    @param searchTimeOut the search timeout in seconds
    @return vector of BPNGDevice
    */
    std::vector<OnlineLoggerInfo> getLoggerList(unsigned searchTimeOut);

    /**
    Checks a vector of BPNGDevice for TSL chains. Combines the devices with same tslNetworkId (except -1)
    to TSLCluster.

    @param loggersInNetwork the BPNGDevice in network found by getLoggerList(unsigned searchTimeOut)
    @return vector of TSLCluster
    */
    std::vector<TSLCluster> getTSLs(std::vector<OnlineLoggerInfo> loggersInNetwork);

    // listener function for observing blue PiraT 2 devices in network
    virtual void WINAPI onBPNGDeviceDetected(OnlineLoggerInfo *info);

    virtual void WINAPI onBPNGDeviceDisappeared(OnlineLoggerInfo *info);

    virtual void WINAPI onBPNGDeviceStateChange(OnlineLoggerInfo *info);

    // listener function not needed by BPNGLoggerDetector
    virtual int WINAPI onProgressDataDownload(int percentCompleted)
    {
        return 1;
    }

    virtual int WINAPI onProgressConversion(int percentCompleted, const char *status)
    {
        return 1;
    }

    virtual void WINAPI onStatusMessage(const char* statusMsg) {}

    virtual void WINAPI onWarning(BPNGWarningCode warningCode, const char *warnMsg) {}

    virtual int WINAPI onTargetPathTooLong(char *newTarget, int maxSize)
    {
        return 0;
    }

    virtual int WINAPI getOverwritingPermission(const char *filePath)
    {
        return 1;
    }

    virtual const char* WINAPI onLogInDataRequired(unsigned mbnr)
    {
        return NULL;
    }

    virtual void WINAPI onInvalidPwConfigFound(unsigned mbnr) {}

    virtual void WINAPI onLogInDataFailed() {}

    virtual void WINAPI onResetLogInDataFailed() {}

    virtual void WINAPI onFuncAccessDenied() {}

    virtual int WINAPI onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char *drive)
    {
        return 1;
    }

    virtual void WINAPI onFirmwareUpdateProgress(int percentage, int stepId, int subStepId, const char *desc) {}

    virtual void WINAPI onFirmwareUpdateError(int errorId) {}

    virtual int WINAPI onGetLogReportProgress(int percentage, const char *desc)
    {
        return 1;
    }

    virtual int WINAPI onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char *drive, const char *msg)
    {
        return 1;
    }

    virtual void WINAPI onDownloadStart(int64_t totalAmountOfBytes) {}

    virtual void WINAPI onConversionStart(int64_t totalAmountOfBytes) {}
    
    virtual const char* WINAPI onExtractionPasswordRequired(uint8_t retryCount) { return ""; }


private:
    OnlineLoggerInfo createCopy(OnlineLoggerInfo *from);
    void freeCopy(OnlineLoggerInfo who);

    std::vector<OnlineLoggerInfo> m_loggerList;
};

#endif  // BPNGLOGGERDETECTOR_HH
