//******************************************************************************
//******************************************************************************
/**
* @file    blue_PiraT2_console_converter.cpp
* @author  Fred Ndongmo-Mbeumo
* @date    12.05.2013
*
* @brief   Sample project for the blue PiraT 2 Client library
*/
// $Id$
//******************************************************************************
//******************************************************************************
#include "BPNGDefines.h"
#include "IBPNGClient.h"
#include "IBPNGClientListener.h"
#include "BPNGLoggerDetector.hh"
#include "RdbEventList.hh"
#include <iostream>
#include <direct.h>
#include <errno.h>
#include <time.h>
#include <fstream>
#include <sys/stat.h>
#include <sstream>
#include <map>

using namespace std;

class ConversionListener : IBPNGClientListener
{
public:
    virtual void WINAPI onBPNGDeviceDetected(OnlineLoggerInfo* info) {}
    virtual void WINAPI onBPNGDeviceDisappeared(OnlineLoggerInfo* info) {}
    virtual void WINAPI onBPNGDeviceStateChange(OnlineLoggerInfo* info) {}
    virtual int WINAPI onProgressDataDownload(int percentCompleted) {
        return 1;
    }
    virtual void WINAPI onStatusMessage(const char* statusMsg) {}
    virtual void WINAPI onWarning(BPNGWarningCode warningCode, const char* warnMsg) {}
    virtual int WINAPI onTargetPathTooLong(char* newTarget, int maxSize) {
        return 0;
    }
    virtual int WINAPI onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char* drive, const char* msg) {
        return 1;
    }
    virtual void WINAPI onFirmwareUpdateProgress(int percentage, int stepId, int subStepId, const char* desc) {}
    virtual void WINAPI onFirmwareUpdateError(int  errorId) {}
    virtual int WINAPI onGetLogReportProgress(int percentage, const char* desc) {
        return 1;
    }
    
    virtual void WINAPI onDownloadStart(int64_t totalAmountOfBytes) {}
    virtual void WINAPI onConversionStart(int64_t totalAmountOfBytes) {}
    virtual const char* WINAPI onExtractionPasswordRequired(uint8_t retryCount) { return ""; }

    virtual inline int WINAPI onProgressConversion(int percentCompleted, const char* status);
    virtual inline int WINAPI getOverwritingPermission(const char* filePath);

    virtual const char* WINAPI onLogInDataRequired(unsigned mbnr){ return ""; }
    virtual void WINAPI onInvalidPwConfigFound(unsigned mbnr){}
    virtual void WINAPI onLogInDataFailed(){}
    virtual void WINAPI onResetLogInDataFailed(){}
    virtual void WINAPI onFuncAccessDenied(){}

private:
};

inline int WINAPI
ConversionListener::onProgressConversion(int percentCompleted, const char* status)
{
    cout << "Conversion progress: " << percentCompleted << "%";
    if (string(status) != "")
        cout << ", " << status << endl;
    else
        cout << endl;
    return 1;
}

inline int WINAPI
ConversionListener::getOverwritingPermission(const char* filePath)
{
    cout << "Overwriting existing file: " << filePath << endl;
    return 1;
}


int convert(string offlinePath, string propertiesPath, string formatPath, string targetPath, map<int, string> dbcAssignment);
std::map<int, string> readCanDataBasesIniFile(string dbcIniFilePath);

string helpText =
"blue_PiraT2_console_converter.exe - client lib version " + string(BPNG_CLIENT_LIB_VERSION) + "\n"
"This tool is distributed as sample for the usage of the blue PiraT 2 Client library.\n"
"You can find all required source, binary and project files in the blue PiraT 2 library package.\n\n"
"Converts all trace data of selected channels in a specified data set to the specified formats.\n"
"\nUsage:\nblue_PiraT2_console_converter.exe /I=<path> /F=<file path> [/P=<file path>] [/O=<target directory path>] [/D=<file path>]\n"
"\n"
"  /I\tPath to the blue PiraT 2 offline data set. May be a folder or zip.\n"
"  /P\tPath to an ini file with the general client properties, like e.g. 'name of tester', 'time stamp format', etc.\n"
"    \tA sample ini file is located in the bin directory of the blue PiraT 2 library package.\n"
"  /F\tPath to an ini file with channel/format assignments\n"
"    \tA sample ini file is located in the bin directory of the blue PiraT 2 library package.\n"
"  /O\tPath to the target directory where the converted files will be stored. Default is cwd.\n"
"  /D\tPath to an ini file with can database assignments. Default is cwd.\n"
"    \tA sample ini file (candatabases.ini) is located in the bin directory of the blue PiraT 2 library package.\n";


int main(int argc, char* argv[])
{
    if (argc == 1)
    {
        cout << "ERROR: missing arguments" << endl << endl;
        cout << helpText << endl;
        return -1;
    }

    string offlinePath = "", propertiesPath = "", formatPath = "", targetDir = "", dbcIniFilePath = "";
    map<int, string> dbcAssignment;

    size_t slash, equal;
    for (unsigned i = 1; i < argc; ++i)
    {
        string parameter = (const char*)argv[i];
        slash = parameter.find("/");
        equal = parameter.find("=");
        char opt;
        if (slash != string::npos && slash != parameter.size() - 1)
        {
            opt = parameter[slash + 1];
            if (opt == 'h' || opt == 'H' || opt == '?')
            {
                cout << helpText << endl;
                return 0;
            }
        }

        if (slash != string::npos && equal != string::npos)
        {
            if (opt == 'I' || opt == 'i')
                offlinePath = parameter.substr(equal + 1);
            else if (opt == 'P' || opt == 'p')
                propertiesPath = parameter.substr(equal + 1);
            else if (opt == 'F' || opt == 'f')
                formatPath = parameter.substr(equal + 1);
            else if (opt == 'O' || opt == 'o')
                targetDir = parameter.substr(equal + 1);
            else if (opt == 'D' || opt == 'd')
                dbcIniFilePath = parameter.substr(equal + 1);
        }
    }

    if (offlinePath == "" || formatPath == "")
    {
        cout << "ERROR: missing arguments" << endl;
        cout << helpText << endl;
        return -1;
    }

    if (offlinePath != "")
    {
        struct stat attr;
        if (stat(offlinePath.c_str(), &attr) != 0)
        {
            cout << "ERROR: Can not find: " << offlinePath << endl;
            return -1;
        }
    }

    if (propertiesPath != "")
    {
        struct stat attr;
        if (stat(propertiesPath.c_str(), &attr) != 0)
        {
            cout << "ERROR: Can not find: " << propertiesPath << endl;
            return -1;
        }
    }

    if (formatPath != "")
    {
        struct stat attr;
        if (stat(formatPath.c_str(), &attr) != 0)
        {
            cout << "ERROR: Can not find: " << formatPath << endl;
            return -1;
        }

    }

    if (dbcIniFilePath != "")
    {
        struct stat attr;
        if (stat(dbcIniFilePath.c_str(), &attr) != 0)
        {
            cout << "ERROR: Can not find: " << dbcIniFilePath << endl;
            return -1;
        }
        dbcAssignment = readCanDataBasesIniFile(dbcIniFilePath);
    }

    if (targetDir == "")
    {
        cout << "Missing target directory, use current working directory instead:" << endl;
        char buffer[300];
        memset(buffer, '\0', 300);
        _getcwd(buffer, 300);
        targetDir = string(buffer);
        cout << targetDir << endl;
    }

    return convert(offlinePath.c_str(), propertiesPath.c_str(), formatPath.c_str(), targetDir.c_str(), dbcAssignment);
}

void removeQuotes(string& str)
{
    if (str.size() && (str[0] == '"' || str[0] == '\''))
        str = str.substr(1);

    if (str.size() && (str[str.size() - 1] == '"' || str[str.size() - 1] == '\''))
        str = str.substr(0, str.size() - 1);
}

int convert(string offlinePath, string propertiesPath, string formatPath, string targetPath, map<int, string> dbcAssignment)
{
    removeQuotes(offlinePath);
    removeQuotes(propertiesPath);
    removeQuotes(formatPath);
    removeQuotes(targetPath);

    IBPNGClient* client = getBPNGClient();

    ConversionListener listener;
    client->addListener((IBPNGClientListener*)&listener);

    // The Offline will be checked and the created Client will be initialise with perpertries of Offlinedaten.
    bool ret = client->initOffline(offlinePath.c_str());

    if (ret == 0)
    {
        BPNGError err = client->getLastError();
        cout << "Failed to init offline." << endl;
        cout << "BPNGErrCode: " << err.code << ", " << err.msg << endl;
        client->release();
        return -1;
    }

    // due to this function,properties of convertion will added to the Client.
    client->loadProperties(propertiesPath.c_str());

    // Ensure the out directory exists
    ret = _mkdir(targetPath.c_str());
    if (ret != 0 && errno != EEXIST)
    {
        cout << "Failed to create output directory" << endl;
        client->release();
        return -1;
    }

    //assign client with dbc files
    if (!dbcAssignment.empty())
    {
        for (map<int, string>::iterator iter = dbcAssignment.begin(); iter != dbcAssignment.end(); ++iter)
        {
            cout << "[INFO] Assign [" << (*iter).second << "] to CAN #" << (*iter).first << endl;
            client->assignDBCFile((*iter).first - 1, (*iter).second.c_str());
        }
    }

    // Get a conversion set
    IConversionSet* conversionSet = client->createNewConversionSet();

    //range of the convertion
    conversionSet->addRdbIdRange(1, MAXUINT64);

    // due to this function the used Channels and output formats are setting
    conversionSet->loadFormats(formatPath.c_str());

    ret = client->convertData(conversionSet, targetPath.c_str());
    if (ret == 0)
    {
        BPNGError err = client->getLastError();
        cout << "Failed to convert data." << endl;
        cout << "BPNGErrCode: " << err.code << ", " << err.msg << endl;
        client->release();
        return -1;
    }
    // free memory
    client->release();
    return 0;
}

//parse dbc assignment ini file, return assignment map
map<int, string> readCanDataBasesIniFile(string dbcIniFilePath)
{
    map<int, string> retVal;
    ifstream input(dbcIniFilePath.c_str());
    string line;
    while (getline(input, line))
    {
        if (line[0] == '#')
        {
            //ignore comment lines
            continue;
        }

        int equalPos = line.find("=");
        if (equalPos == string::npos)
        {
            continue;
        }

        int channelIndex = atoi(line.substr(0, equalPos).c_str());
        string path = line.substr(equalPos + 1);
        retVal.insert(make_pair(channelIndex, path));
    }

    return retVal;
}